@@ -9,15 +9,13 @@ Test basic changelog functionality: Ansible collections
 
 import os
 
-from typing import List, Optional
-
 import mock
 
-from antsibull_changelog.config import PathsConfig
-
 from fixtures import collection_changelog  # noqa: F401; pylint: disable=unused-variable
 from fixtures import create_plugin
 
+import antsibull_changelog.ansible  # noqa: F401; pylint: disable=unused-variable
+
 
 def test_changelog_init(  # pylint: disable=redefined-outer-name
         collection_changelog):  # noqa: F811
@@ -1319,210 +1317,284 @@ foo
 ''')
 
 
-def fake_ansible_doc_collection(paths: PathsConfig,
-                                playbook_dir: Optional[str],
-                                plugin_type: str,
-                                plugin_names: List[str]) -> dict:
-    """
-    Fake ansible-doc mock for collection.
-    """
-    if plugin_type == 'callback':
-        assert sorted(plugin_names) == ['acme.test.test_callback']
-        return {
-            'acme.test.test_callback': {
-                'doc': {
-                    'author': ['Someone else'],
-                    'description': ['This is a relatively new callback added before.'],
-                    'filename': os.path.join(paths.base_dir, 'plugins',
-                                             'callback', 'test_callback.py'),
-                    'name': 'test_callback',
-                    'options': {},
-                    'short_description': 'A not so old callback',
-                    'version_added': '0.5.0',
-                },
-                'examples': '# Some examples\n',
-                'metadata': {
-                    'status': ['preview'],
-                    'supported_by': 'community',
-                },
-                'return': {},
-            }
-        }
-    if plugin_type == 'module':
-        assert sorted(plugin_names) == ['acme.test.cloud.sky.old_module', 'acme.test.test_module']
-        return {
-            'acme.test.cloud.sky.old_module': {
-                'doc': {
-                    'author': ['Elder'],
-                    'description': ['This is an old module.'],
-                    'filename': os.path.join(paths.base_dir, 'plugins', 'modules',
-                                             'cloud', 'sky', 'old_module.py'),
-                    'name': 'old_module',
-                    'options': {},
-                    'short_description': 'An old module',
-                },
-                'examples': '# Some examples\n',
-                'metadata': {
-                    'status': ['preview'],
-                    'supported_by': 'community',
-                },
-                'return': {},
+FAKE_PLUGINS = {
+    'callback': {
+        'acme.test.test_callback': {
+            'doc': {
+                'author': ['Someone else'],
+                'description': ['This is a relatively new callback added before.'],
+                'filename': os.path.join('plugins', 'callback', 'test_callback.py'),
+                'name': 'test_callback',
+                'options': {},
+                'short_description': 'A not so old callback',
+                'version_added': '0.5.0',
             },
-            'acme.test.test_module': {
-                'doc': {
-                    'author': ['Someone'],
-                    'description': ['This is a test module.'],
-                    'filename': os.path.join(paths.base_dir, 'plugins',
-                                             'modules', 'test_module.py'),
-                    'name': 'test_module',
-                    'options': {},
-                    'short_description': 'A test module',
+            'examples': '# Some examples\n',
+            'metadata': {
+                'status': ['preview'],
+                'supported_by': 'community',
+            },
+            'return': {},
+        },
+    },
+    'module': {
+        'acme.test.cloud.sky.old_module': {
+            'doc': {
+                'author': ['Elder'],
+                'description': ['This is an old module.'],
+                'filename': os.path.join('plugins', 'modules', 'cloud', 'sky', 'old_module.py'),
+                'name': 'old_module',
+                'options': {},
+                'short_description': 'An old module',
+            },
+            'examples': '# Some examples\n',
+            'metadata': {
+                'status': ['preview'],
+                'supported_by': 'community',
+            },
+            'return': {},
+        },
+        'acme.test.test_module': {
+            'doc': {
+                'author': ['Someone'],
+                'description': ['This is a test module.'],
+                'filename': os.path.join('plugins', 'modules', 'test_module.py'),
+                'name': 'test_module',
+                'options': {},
+                'short_description': 'A test module',
+                'version_added': '1.0.0',
+            },
+            'examples': '',
+            'metadata': {
+                'status': ['preview'],
+                'supported_by': 'community',
+            },
+            'return': {},
+        },
+    },
+    'role': {
+        'acme.test.test_role': {
+            'collection': 'acme.test',
+            'entry_points': {
+                'main': {
+                    'short_description': 'Test role',
                     'version_added': '1.0.0',
+                    'options': {},
                 },
-                'examples': '',
-                'metadata': {
-                    'status': ['preview'],
-                    'supported_by': 'community',
+                'foo': {
+                    'short_description': 'Test role foo entrypoint',
+                    'version_added': '0.9.0',
+                    'options': {},
                 },
-                'return': {},
-            }
-        }
-    raise Exception(
-        'ansible-doc should not be called with plugin_type == "{0}"'.format(plugin_type))
+            },
+            'path': '',
+        },
+        'acme.test.old_role': {
+            'collection': 'acme.test',
+            'entry_points': {
+                'main': {
+                    'short_description': 'Old role',
+                    'options': {},
+                },
+            },
+            'path': '',
+        },
+        'acme.test.funky_role': {
+            'collection': 'acme.test',
+            'entry_points': {
+                'funky': {
+                    'short_description': 'A funky role',
+                    'version_added': '1.0.0',
+                    'options': {},
+                },
+            },
+            'path': '',
+        },
+    },
+}
 
 
-@mock.patch('antsibull_changelog.plugins.run_ansible_doc', fake_ansible_doc_collection)
+class FakeAnsibleRelease:
+    def __init__(self, version: str, codename: str):
+        self.__version__ = version
+        self.__codename__ = codename
+
+
+@mock.patch('antsibull_changelog.ansible.HAS_ANSIBLE_RELEASE', True)
+@mock.patch('antsibull_changelog.ansible.ansible_release', FakeAnsibleRelease('2.11.0', 'dummy codename'))
 def test_changelog_release_plugin_cache(  # pylint: disable=redefined-outer-name
         collection_changelog):  # noqa: F811
-    collection_changelog.set_galaxy({
-        'version': '1.0.0',
-    })
-    collection_changelog.config.title = 'My Amazing Collection'
-    collection_changelog.set_config(collection_changelog.config)
-    collection_changelog.add_fragment_line(
-        '1.0.0.yml', 'release_summary', 'This is the first proper release.')
-    collection_changelog.add_plugin('module', 'test_module.py', create_plugin(
-        DOCUMENTATION={
-            'name': 'test_module',
-            'short_description': 'A test module',
-            'version_added': '1.0.0',
-            'description': ['This is a test module.'],
-            'author': ['Someone'],
-            'options': {},
-        },
-        EXAMPLES='',
-        RETURN={},
-    ))
-    collection_changelog.add_plugin('module', '__init__.py', create_plugin(
-        DOCUMENTATION={
-            'name': 'bad_module',
-            'short_description': 'Bad module',
-            'description': ['This should be ignored, not found as a module!.'],
-            'author': ['badguy'],
-            'options': {},
-        },
-        EXAMPLES='# Some examples\n',
-        RETURN={},
-    ), subdirs=['cloud'])
-    collection_changelog.add_plugin('module', 'old_module.py', create_plugin(
-        DOCUMENTATION={
-            'name': 'old_module',
-            'short_description': 'An old module',
-            'description': ['This is an old module.'],
-            'author': ['Elder'],
-            'options': {},
-        },
-        EXAMPLES='# Some examples\n',
-        RETURN={},
-    ), subdirs=['cloud', 'sky'])
-    collection_changelog.add_plugin('module', 'bad_module2', create_plugin(
-        DOCUMENTATION={
-            'name': 'bad_module2',
-            'short_description': 'An bad module',
-            'description': ['Shold not be found either.'],
-            'author': ['Elder'],
-            'options': {},
-        },
-        EXAMPLES='# Some examples\n',
-        RETURN={},
-    ), subdirs=['cloud', 'sky'])
-    collection_changelog.add_plugin('callback', 'test_callback.py', create_plugin(
-        DOCUMENTATION={
-            'name': 'test_callback',
-            'short_description': 'A not so old callback',
-            'version_added': '0.5.0',
-            'description': ['This is a relatively new callback added before.'],
-            'author': ['Someone else'],
-            'options': {},
-        },
-        EXAMPLES='# Some examples\n',
-        RETURN={},
-    ))
-    collection_changelog.add_plugin('callback', 'test_callback2.py', create_plugin(
-        DOCUMENTATION={
-            'name': 'test_callback2',
-            'short_description': 'This one should not be found.',
-            'version_added': '2.9',
-            'description': ['This is a relatively new callback added before.'],
-            'author': ['Someone else'],
-            'options': {},
-        },
-        EXAMPLES='# Some examples\n',
-        RETURN={},
-    ), subdirs=['dont', 'find', 'me'])
+    with mock.patch('subprocess.check_output',
+                    collection_changelog.create_fake_subprocess_ansible_doc(FAKE_PLUGINS)):
+        collection_changelog.set_galaxy({
+            'version': '1.0.0',
+        })
+        collection_changelog.config.title = 'My Amazing Collection'
+        collection_changelog.set_config(collection_changelog.config)
+        collection_changelog.add_fragment_line(
+            '1.0.0.yml', 'release_summary', 'This is the first proper release.')
+        collection_changelog.add_plugin('module', 'test_module.py', create_plugin(
+            DOCUMENTATION={
+                'name': 'test_module',
+                'short_description': 'A test module',
+                'version_added': '1.0.0',
+                'description': ['This is a test module.'],
+                'author': ['Someone'],
+                'options': {},
+            },
+            EXAMPLES='',
+            RETURN={},
+        ))
+        collection_changelog.add_plugin('module', '__init__.py', create_plugin(
+            DOCUMENTATION={
+                'name': 'bad_module',
+                'short_description': 'Bad module',
+                'description': ['This should be ignored, not found as a module!.'],
+                'author': ['badguy'],
+                'options': {},
+            },
+            EXAMPLES='# Some examples\n',
+            RETURN={},
+        ), subdirs=['cloud'])
+        collection_changelog.add_plugin('module', 'old_module.py', create_plugin(
+            DOCUMENTATION={
+                'name': 'old_module',
+                'short_description': 'An old module',
+                'description': ['This is an old module.'],
+                'author': ['Elder'],
+                'options': {},
+            },
+            EXAMPLES='# Some examples\n',
+            RETURN={},
+        ), subdirs=['cloud', 'sky'])
+        collection_changelog.add_plugin('module', 'bad_module2', create_plugin(
+            DOCUMENTATION={
+                'name': 'bad_module2',
+                'short_description': 'An bad module',
+                'description': ['Shold not be found either.'],
+                'author': ['Elder'],
+                'options': {},
+            },
+            EXAMPLES='# Some examples\n',
+            RETURN={},
+        ), subdirs=['cloud', 'sky'])
+        collection_changelog.add_plugin('callback', 'test_callback.py', create_plugin(
+            DOCUMENTATION={
+                'name': 'test_callback',
+                'short_description': 'A not so old callback',
+                'version_added': '0.5.0',
+                'description': ['This is a relatively new callback added before.'],
+                'author': ['Someone else'],
+                'options': {},
+            },
+            EXAMPLES='# Some examples\n',
+            RETURN={},
+        ))
+        collection_changelog.add_plugin('callback', 'test_callback2.py', create_plugin(
+            DOCUMENTATION={
+                'name': 'test_callback2',
+                'short_description': 'This one should not be found.',
+                'version_added': '2.9',
+                'description': ['This is a relatively new callback added before.'],
+                'author': ['Someone else'],
+                'options': {},
+            },
+            EXAMPLES='# Some examples\n',
+            RETURN={},
+        ), subdirs=['dont', 'find', 'me'])
+        collection_changelog.add_role('test_role', {
+            'main': {
+                'short_description': 'Test role',
+                'version_added': '1.0.0',
+                'options': {},
+            },
+            'foo': {
+                'short_description': 'Test role foo entrypoint',
+                'version_added': '0.9.0',
+                'options': {},
+            },
+        })
+        collection_changelog.add_role('old_role', {
+            'main': {
+                'short_description': 'Old role',
+                'options': {},
+            },
+        })
+        collection_changelog.add_role('funky_role', {
+            'funky': {
+                'short_description': 'A funky role',
+                'version_added': '1.0.0',
+                'options': {},
+            },
+        })
 
-    assert collection_changelog.run_tool('release', ['-v', '--date', '2020-01-02']) == 0
+        assert collection_changelog.run_tool('release', ['-v', '--date', '2020-01-02']) == 0
 
-    diff = collection_changelog.diff()
-    assert diff.added_dirs == []
-    assert diff.added_files == [
-        'CHANGELOG.rst',
-        'changelogs/.plugin-cache.yaml',
-        'changelogs/changelog.yaml',
-    ]
-    assert diff.removed_dirs == []
-    assert diff.removed_files == ['changelogs/fragments/1.0.0.yml']
-    assert diff.changed_files == []
+        diff = collection_changelog.diff()
+        assert diff.added_dirs == []
+        assert diff.added_files == [
+            'CHANGELOG.rst',
+            'changelogs/.plugin-cache.yaml',
+            'changelogs/changelog.yaml',
+        ]
+        assert diff.removed_dirs == []
+        assert diff.removed_files == ['changelogs/fragments/1.0.0.yml']
+        assert diff.changed_files == []
 
-    plugin_cache = diff.parse_yaml('changelogs/.plugin-cache.yaml')
-    assert plugin_cache['version'] == '1.0.0'
+        plugin_cache = diff.parse_yaml('changelogs/.plugin-cache.yaml')
+        assert plugin_cache['version'] == '1.0.0'
 
-    # Plugin cache: modules
-    assert sorted(plugin_cache['plugins']['module']) == ['old_module', 'test_module']
-    assert plugin_cache['plugins']['module']['old_module']['name'] == 'old_module'
-    assert plugin_cache['plugins']['module']['old_module']['namespace'] == 'cloud.sky'
-    assert plugin_cache['plugins']['module']['old_module']['description'] == 'An old module'
-    assert plugin_cache['plugins']['module']['old_module']['version_added'] is None
-    assert plugin_cache['plugins']['module']['test_module']['name'] == 'test_module'
-    assert plugin_cache['plugins']['module']['test_module']['namespace'] == ''
-    assert plugin_cache['plugins']['module']['test_module']['description'] == 'A test module'
-    assert plugin_cache['plugins']['module']['test_module']['version_added'] == '1.0.0'
+        # Plugin cache: modules
+        assert sorted(plugin_cache['plugins']['module']) == ['old_module', 'test_module']
+        assert plugin_cache['plugins']['module']['old_module']['name'] == 'old_module'
+        assert plugin_cache['plugins']['module']['old_module']['namespace'] == 'cloud.sky'
+        assert plugin_cache['plugins']['module']['old_module']['description'] == 'An old module'
+        assert plugin_cache['plugins']['module']['old_module']['version_added'] is None
+        assert plugin_cache['plugins']['module']['test_module']['name'] == 'test_module'
+        assert plugin_cache['plugins']['module']['test_module']['namespace'] == ''
+        assert plugin_cache['plugins']['module']['test_module']['description'] == 'A test module'
+        assert plugin_cache['plugins']['module']['test_module']['version_added'] == '1.0.0'
 
-    # Plugin cache: callbacks
-    assert sorted(plugin_cache['plugins']['callback']) == ['test_callback']
-    assert plugin_cache['plugins']['callback']['test_callback']['name'] == 'test_callback'
-    assert plugin_cache['plugins']['callback']['test_callback']['description'] == \
-        'A not so old callback'
-    assert plugin_cache['plugins']['callback']['test_callback']['version_added'] == '0.5.0'
-    assert 'namespace' not in plugin_cache['plugins']['callback']['test_callback']
+        # Plugin cache: callbacks
+        assert sorted(plugin_cache['plugins']['callback']) == ['test_callback']
+        assert plugin_cache['plugins']['callback']['test_callback']['name'] == 'test_callback'
+        assert plugin_cache['plugins']['callback']['test_callback']['description'] == \
+            'A not so old callback'
+        assert plugin_cache['plugins']['callback']['test_callback']['version_added'] == '0.5.0'
+        assert 'namespace' not in plugin_cache['plugins']['callback']['test_callback']
 
-    # Changelog
-    changelog = diff.parse_yaml('changelogs/changelog.yaml')
-    assert changelog['ancestor'] is None
-    assert sorted(changelog['releases']) == ['1.0.0']
-    assert changelog['releases']['1.0.0']['release_date'] == '2020-01-02'
-    assert changelog['releases']['1.0.0']['changes'] == {
-        'release_summary': 'This is the first proper release.'
-    }
-    assert changelog['releases']['1.0.0']['fragments'] == ['1.0.0.yml']
-    assert len(changelog['releases']['1.0.0']['modules']) == 1
-    assert changelog['releases']['1.0.0']['modules'][0]['name'] == 'test_module'
-    assert changelog['releases']['1.0.0']['modules'][0]['namespace'] == ''
-    assert changelog['releases']['1.0.0']['modules'][0]['description'] == 'A test module'
-    assert 'version_added' not in changelog['releases']['1.0.0']['modules'][0]
+        # Plugin cache: roles
+        assert sorted(plugin_cache['objects']['role']) == ['funky_role', 'old_role', 'test_role']
+        assert plugin_cache['objects']['role']['funky_role']['name'] == 'funky_role'
+        assert plugin_cache['objects']['role']['funky_role']['description'] is None
+        assert plugin_cache['objects']['role']['funky_role']['version_added'] is None
+        assert 'namespace' not in plugin_cache['objects']['role']['funky_role']
+        assert plugin_cache['objects']['role']['old_role']['name'] == 'old_role'
+        assert plugin_cache['objects']['role']['old_role']['description'] == 'Old role'
+        assert plugin_cache['objects']['role']['old_role']['version_added'] is None
+        assert 'namespace' not in plugin_cache['objects']['role']['old_role']
+        assert plugin_cache['objects']['role']['test_role']['name'] == 'test_role'
+        assert plugin_cache['objects']['role']['test_role']['description'] == 'Test role'
+        assert plugin_cache['objects']['role']['test_role']['version_added'] == '1.0.0'
+        assert 'namespace' not in plugin_cache['objects']['role']['test_role']
 
-    assert diff.file_contents['CHANGELOG.rst'].decode('utf-8') == (
-        r'''===================================
+        # Changelog
+        changelog = diff.parse_yaml('changelogs/changelog.yaml')
+        assert changelog['ancestor'] is None
+        assert sorted(changelog['releases']) == ['1.0.0']
+        assert changelog['releases']['1.0.0']['release_date'] == '2020-01-02'
+        assert changelog['releases']['1.0.0']['changes'] == {
+            'release_summary': 'This is the first proper release.'
+        }
+        assert changelog['releases']['1.0.0']['fragments'] == ['1.0.0.yml']
+        assert len(changelog['releases']['1.0.0']['modules']) == 1
+        assert changelog['releases']['1.0.0']['modules'][0]['name'] == 'test_module'
+        assert changelog['releases']['1.0.0']['modules'][0]['namespace'] == ''
+        assert changelog['releases']['1.0.0']['modules'][0]['description'] == 'A test module'
+        assert 'version_added' not in changelog['releases']['1.0.0']['modules'][0]
+
+        assert diff.file_contents['CHANGELOG.rst'].decode('utf-8') == (
+            r'''===================================
 My Amazing Collection Release Notes
 ===================================
 
@@ -1541,4 +1613,16 @@ New Modules
 -----------
 
 - acme.test.test_module - A test module
+
+New Roles
+---------
+
+- acme.test.test_role - Test role
 ''')
+
+        # Force reloading plugins. This time use ansible-doc for listing plugins.
+        assert collection_changelog.run_tool('generate', ['-v', '--reload-plugins', '--use-ansible-doc']) == 0
+
+        diff = collection_changelog.diff()
+        diff.dump()
+        assert diff.unchanged
